# $Id: V.py,v 1.2 2005-07-14 01:36:41 gosselin_a Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1  2004/08/02 15:36:04  gosselin
# Initial revision
#
# Author: Andre Gosselin
#         Maurice-Lamontagne Institute
#         gosselina@dfo-mpo.gc.ca

"""
V (Vgroup) API (:mod:`pyhdf.V`)
===============================

A module of the pyhdf package implementing the V (Vgroup)
API of the NCSA HDF4 library.

Introduction
------------
V is one of the modules composing pyhdf, a python package implementing
the NCSA HDF library and letting one manage HDF files from within a python
program. Two versions of the HDF library currently exist, version 4 and
version 5. pyhdf only implements version 4 of the library. Many
different APIs are to be found inside the HDF4 specification.
Currently, pyhdf implements just a few of those: the SD, VS and V APIs.
Other APIs should be added in the future (GR, AN, etc).

The V API supports the definition of vgroups inside an HDF file. A vgroup
can thought of as a collection of arbitrary "references" to other HDF
objects defined in the same file. A vgroup may hold references to
other vgroups. It is thus possible to organize HDF objects into some sort
of a hierarchy, similar to files grouped into a directory tree under unix.
This vgroup hierarchical nature partly explains the origin of the "HDF"
name (Hierarchical Data Format). vgroups can help logically organize the
contents of an HDF file, for example by grouping together all the datasets
belonging to a given experiment, and subdividing those datasets according
to the day of the experiment, etc.

The V API provides functions to find and access an existing vgroup,
create a new one, delete a vgroup, identify the members of a vgroup, add
and remove members to and from a vgroup, and set and query attributes
on a vgroup. The members of a vgroup are identified through their tags
and reference numbers. Tags are constants identifying each main object type
(dataset, vdata, vgroup). Reference numbers serve to distinguish among
objects of the same type. To add an object to a vgroup, one must first
initialize that object using the API proper to that object type (eg: SD for
a dataset) so as to create a reference number for that object, and then
pass this reference number and the type tag to the V API. When reading the
contents of a vgroup, the V API returns the tags and reference numbers of
the objects composing the vgroup. The user program must then call the
proper API to process each object, based on tag of this object (eg: VS for
a tag identifying a vdata object).

Some limitations of the V API must be stressed. First, HDF imposes
no integrity constraint whatsoever on the contents of a vgroup, nor does it
help maintain such integrity. For example, a vgroup is not strictly
hierarchical, because an object can belong to more than one vgroup. It would
be easy to create vgroups showing cycles among their members. Also, a vgroup
member is simply a reference to an HDF object. If this object is afterwards
deleted for any reason, the vgroup membership will not be automatically
updated. The vgroup will refer to a non-existent object and thus be left
in an inconsistent state. Nothing prevents adding the same member more than
once to a vgroup, and giving the same name to more than one vgroup.
Finally, the HDF library seems to make heavy use of vgroups for its own
internal needs, and creates vgroups "behind the scenes". This may make it
difficult to pick up "user defined" vgroups when browsing an HDF file.

Accessing the V module
-----------------------
To access the V module a python program can say one of:

  >>> import pyhdf.V           # must prefix names with "pyhdf.V."
  >>> from pyhdf import V      # must prefix names with "V."
  >>> from pyhdf.V import *    # names need no prefix

This document assumes the last import style is used.

V is not self-contained, and needs functionality provided by another
pyhdf module, namely the HDF module. This module must thus be imported
also:

  >>> from .HDF import *


Package components
------------------
pyhdf is a proper Python package, eg a collection of modules stored under
a directory whose name is that of the package and which stores an
__init__.py file. Following the normal installation procedure, this
directory will be <python-lib>/site-packages/pyhdf', where <python-lib>
stands for the python installation directory.

For each HDF API exists a corresponding set of modules.

The following modules are related to the V API.

  _hdfext
    C extension module responsible for wrapping the HDF
    C library for all python modules
  hdfext
    python module implementing some utility functions
    complementing the _hdfext extension module
  error
    defines the HDF4Error exception
  HDF
    python module providing support to the V module
  V
    python module wrapping the V API routines inside
    an OOP framework

_hdfext and hdfext were generated using the SWIG preprocessor.
SWIG is however *not* needed to run the package. Those two modules
are meant to do their work in the background, and should never be called
directly. Only HDF and V should be imported by the user program.


Prerequisites
-------------
The following software must be installed in order for the V module to
work.

  HDF (v4) library
    pyhdf does *not* include the HDF4 library, which must
    be installed separately.

    HDF is available at:
    "https://portal.hdfgroup.org/display/support/Download+HDF4".

Numeric is also needed by the SD module. See the SD module documentation.


Summary of differences between the pyhdf and C V API
-----------------------------------------------------
Most of the differences between the pyhdf and C V API can
be summarized as follows.

   -In the C API, every function returns an integer status code, and values
    computed by the function are returned through one or more pointers
    passed as arguments.
   -In pyhdf, error statuses are returned through the Python exception
    mechanism, and values are returned as the method result. When the
    C API specifies that multiple values are returned, pyhdf returns a
    sequence of values, which are ordered similarly to the pointers in the
    C function argument list.

Error handling
--------------
All errors reported by the C V API with a SUCCESS/FAIL error code
are reported by pyhdf using the Python exception mechanism.
When the C library reports a FAIL status, pyhdf raises an HDF4Error
exception (a subclass of Exception) with a descriptive message.
Unfortunately, the C library is rarely informative about the cause of
the error. pyhdf does its best to try to document the error, but most
of the time cannot do more than saying "execution error".

V needs support from the HDF module
------------------------------------
The VS module is not self-contained (countrary to the SD module).
It requires help from the HDF module, namely:

- the HDF.HDF class to open and close the HDF file, and initialize the
  V interface
- the HDF.HC class to provide different sorts of constants (opening modes,
  data types, etc).

A program wanting to access HDF vgroups will almost always need to execute
the following minimal set of calls:

  >>> from pyhdf.HDF import *
  >>> from pyhdf.V import *
  >>> hdfFile = HDF(name, HC.xxx)# open HDF file
  >>> v = hdfFile.vgstart()      # initialize V interface on HDF file
  >>> ...                        # manipulate vgroups
  >>> v.end()                    # terminate V interface
  >>> hdfFile.close()            # close HDF file


Classes summary
---------------

pyhdf wraps the V API using the following python classes::

  V      HDF V interface
  VG     vgroup
  VGAttr vgroup attribute

In more detail::

  V     The V class implements the V (Vgroup) interface applied to an
        HDF file.

        To instantiate a V class, call the vgstart() method of an
        HDF instance.

        methods:
          constructors
            attach()      open an existing vgroup given its name or its
                          reference number, or create a new vgroup,
                          returning a VG instance for that vgroup
            create()      create a new vgroup, returning a VG instance
                          for that vgroup

          closing the interface
            end()         close the V interface on the HDF file

          deleting a vgroup
            delete()      delete the vgroup identified by its name or
                          its reference number

          searching
            find()        find a vgroup given its name, returning
                          the vgroup reference number
            findclass()   find a vgroup given its class name, returning
                          the vgroup reference number
            getid()       return the reference number of the vgroup
                          following the one with the given reference number

  VG    The VG class encapsulates the functionality of a vgroup.

        To instantiate a VG class, call the attach() or create() methods
        of a V class instance.

          constructors

            attr()        return a VGAttr instance representing an attribute
                          of the vgroup
            findattr()    search the vgroup for a given attribute,
                          returning a VGAttr instance for that attribute

          ending access to a vgroup

            detach()      terminate access to the vgroup

          adding a member to a vgroup

            add()         add to the vgroup the HDF object identified by its
                          tag and reference number
            insert()      insert a vdata or a vgroup in the vgroup, given
                          the vdata or vgroup instance

          deleting a member from a vgroup

            delete()      remove from the vgroup the HDF object identified
                          by the given tag and reference number

          querying vgroup

            attrinfo()    return info about all the vgroup attributes
            inqtagref()   determine if the HDF object with the given
                          tag and reference number belongs to the vgroup
            isvg()        determine if the member with the given reference
                          number is a vgroup object
            isvs()        determine if the member with the given reference
                          number is a vdata object
            nrefs()       return the number of vgroup members with the
                          given tag
            tagref()      get the tag and reference number of a vgroup
                          member, given the index number of that member
            tagrefs()     get the tags and reference numbers of all the
                          vgroup members

  VGAttr The VGAttr class provides methods to set and query vgroup
         attributes.

         To create an instance of this class, call the attr() method
         of a VG instance.

         Remember that vgroup attributes can also be set and queried by
         applying the standard python "dot notation" on a VG instance.

           get attribute value(s)

             get()         obtain the attribute value(s)

           set attribute value(s)

             set()         set the attribute to the given value(s) of the
                           given type, first creating the attribute if
                           necessary

           query attribute info

             info()        retrieve attribute name, data type, order and
                           size

Attribute access: low and high level
------------------------------------
The V API allows setting attributes on vgroups. Attributes can be of many
types (int, float, char) of different bit lengths (8, 16, 32, 64 bits),
and can be single or multi-valued. Values of a multi-valued attribute must
all be of the same type.

Attributes can be set and queried in two different ways. First, given a
VG instance (describing a vgroup object), the attr() method of that instance
is called to create a VGAttr instance representing the wanted attribute
(possibly non existent). The set() method of this VGAttr instance is then
called to define the attribute value, creating it if it does not already
exist. The get() method returns the current attribute value. Here is an
example.

  >>> from pyhdf.HDF import *
  >>> from pyhdf.V import *
  >>> f = HDF('test.hdf', HC.WRITE) # Open file 'test.hdf' in write mode
  >>> v = f.vgstart()            # init vgroup interface
  >>> vg = v.attach('vtest', 1)  # attach vgroup 'vtest' in write mode
  >>> attr = vg.attr('version')  # prepare to define the 'version' attribute
                                 # on the vdata
  >>> attr.set(HC.CHAR8,'1.0')   # set attribute 'version' to string '1.0'
  >>> print(attr.get())           # get and print attribute value
  >>> attr = vg .attr('range')   # prepare to define attribute 'range'
  >>> attr.set(HC.INT32,(-10, 15)) # set attribute 'range' to a pair of ints
  >>> print(attr.get())             # get and print attribute value

  >>> vg.detach()                # "close" the vgroup
  >>> v.end()                    # terminate the vgroup interface
  >>> f.close()                  # close the HDF file

The second way consists of setting/querying an attribute as if it were a
normal python class attribute, using the usual dot notation. Above example
then becomes:

  >>> from pyhdf.HDF import *
  >>> from pyhdf.V import *
  >>> f = HDF('test.hdf', HC.WRITE) # Open file 'test.hdf' in write mode
  >>> v = f.vgstart()            # init vgroup interface
  >>> vg = v.attach('vtest', 1)  # attach vdata 'vtest' in write mode
  >>> vg.version = '1.0'         # create vdata attribute 'version',
                                 # setting it to string '1.0'
  >>> print(vg.version)           # print attribute value
  >>> vg.range = (-10, 15)       # create attribute 'range', setting
                                 # it to the pair of ints (-10, 15)
  >>> print(vg.range)             # print attribute value
  >>> vg.detach()                # "close" the vdata
  >>> v.end()                    # terminate the vdata interface
  >>> f.close()                  # close the HDF file

Note how the dot notation greatly simplifies and clarifies the code.
Some latitude is however lost by manipulating attributes in that way,
because the pyhdf package, not the programmer, is then responsible of
setting the attribute type. The attribute type is chosen to be one of:

  =========== ====================================
  HC.CHAR8    if the attribute value is a string
  HC.INT32    if all attribute values are integers
  HC.FLOAT64  otherwise
  =========== ====================================

The first way of handling attribute values must be used if one wants to
define an attribute of any other type (for ex. 8 or 16 bit integers,
signed or unsigned). Also, only a VDAttr instance gives access to attribute
info, through its info() method.

However, accessing HDF attributes as if they were python attributes raises
an important issue. There must exist a way to assign generic attributes
to the python objects without requiring those attributes to be converted
to HDF attributes. pyhdf uses the following rule: an attribute whose name
starts with an underscore ('_') is either a "predefined" HDF attribute
(see below) or a standard python attribute. Otherwise, the attribute
is handled as an HDF attribute. Also, HDF attributes are not stored inside
the object dictionary: the python dir() function will not list them.

Attribute values can be updated, but it is illegal to try to change the
value type, or the attribute order (number of values). This is important
for attributes holding string values. An attribute initialized with an
'n' character string is simply a character attribute of order 'n' (eg a
character array of length 'n'). If 'vg' is a vgroup and we initialize its
'a1' attribute as 'vg.a1 = "abcdef"', then a subsequent update attempt
like 'vg.a1 = "12"' will fail, because we then try to change the order
of the attribute (from 6 to 2). It is mandatory to keep the length of string
attributes constant.

Predefined attributes
---------------------
The VG class supports predefined attributes to get (and occasionnaly set)
attribute values easily using the usual python "dot notation", without
having to call a class method. The names of predefined attributes all start
with an underscore ('_').

In the following table, the RW column holds an X if the attribute
is read/write.

  VG predefined attributes

    =========== === =========================== ===================
    name        RW  description                 C library routine
    =========== === =========================== ===================
    _class      X   class name                  Vgetclass/Vsetclass
    _name       X   vgroup name                 Vgetname/Vsetname
    _nattrs         number of vgroup attributes Vnattrs
    _nmembers       number of vgroup members    Vntagrefs
    _refnum         vgroup reference number     VQueryref
    _tag            vgroup tag                  VQuerytag
    _version        vgroup version number       Vgetversion
    =========== === =========================== ===================


Programming models
------------------

Creating and initializing a vgroup
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
The following program shows how to create and initialize a vgroup inside
an HDF file. It can serve as a model for any program wanting to create
a vgroup::

    from pyhdf.HDF import *
    from pyhdf.V   import *
    from pyhdf.VS  import *
    from pyhdf.SD  import *

    def vdatacreate(vs, name):

        # Create vdata and define its structure
        vd = vs.create(name,
                   (('partid',HC.CHAR8, 5),       # 5 char string
                    ('description',HC.CHAR8, 10), # 10 char string field
                    ('qty',HC.INT16, 1),          # 1 16 bit int field
                    ('wght',HC.FLOAT32, 1),       # 1 32 bit float
                    ('price',HC.FLOAT32,1)        # 1 32 bit float
                   ))

        # Store records
        vd.write((('Q1234', 'bolt',12, 0.01, 0.05),   # record 1
                  ('B5432', 'brush', 10, 0.4, 4.25),  # record 2
                  ('S7613', 'scissor', 2, 0.2, 3.75)  # record 3
                  ))
        # "close" vdata
        vd.detach()

    def sdscreate(sd, name):

        # Create a simple 3x3 float array.
        sds = sd.create(name, SDC.FLOAT32, (3,3))
        # Initialize array
        sds[:] = ((0,1,2),(3,4,5),(6,7,8))
        # "close" dataset.
        sds.endaccess()

    # Create HDF file
    filename = 'inventory.hdf'
    hdf = HDF(filename, HC.WRITE|HC.CREATE)

    # Initialize the SD, V and VS interfaces on the file.
    sd = SD(filename, SDC.WRITE)  # SD interface
    vs = hdf.vstart()             # vdata interface
    v  = hdf.vgstart()            # vgroup interface

    # Create vdata named 'INVENTORY'.
    vdatacreate(vs, 'INVENTORY')
    # Create dataset named "ARR_3x3"
    sdscreate(sd, 'ARR_3x3')

    # Attach the vdata and the dataset.
    vd = vs.attach('INVENTORY')
    sds = sd.select('ARR_3x3')

    # Create vgroup named 'TOTAL'.
    vg = v.create('TOTAL')

    # Add vdata to the vgroup
    vg.insert(vd)
    # We could also have written this:
    # vgroup.add(vd._tag, vd._refnum)
    # or this:
    # vgroup.add(HC.DFTAG_VH, vd._refnum)

    # Add dataset to the vgroup
    vg.add(HC.DFTAG_NDG, sds.ref())

    # Close vgroup, vdata and dataset.
    vg.detach()       # vgroup
    vd.detach()       # vdata
    sds.endaccess()   # dataset

    # Terminate V, VS and SD interfaces.
    v.end()           # V interface
    vs.end()          # VS interface
    sd.end()          # SD interface

    # Close HDF file.
    hdf.close()

The program starts by defining two functions vdatacreate() and sdscreate(),
which will serve to create the vdata and dataset objects we need. Those
functions are not essential to the example. They simply help to make the
example self-contained. Refer to the VS and SD module documentation for
additional explanations about how these functions work.

After opening the HDF file in write mode, the SD, V and VS interfaces are
initialized on the file. Next vdatacreate() is called to create a new vdata
named 'INVENTORY' on the VS instance, and sdscreate() to create a new
dataset named 'ARR_3x3' on the SD instance. This is done so that we have a
vdata and a dataset to play with.

The vdata and the dataset are then attached ("opened"). The create()
method of the V instance is then called to create a new vgroup named
'TOTAL'. The vgroup is then populated by calling its insert() method to add
the vdata 'INVENTORY', and its add() method to add the 'ARR_3x3' dataset.
Note that insert() is just a commodity method that simplifies adding a
vdata or a vgroup to a vgroup, avoiding the need to pass an object tag and
reference number. There is no such commodity method for adding a dataset
to a vgroup. The dataset must be added by specifying its tag and reference
number. Note that the tags to be used are defined inside the HDF module as
constants of the HC class: DFTAG_NDG for a dataset, DFTAG_VG for a vgroup,
DFTAG_VH for a vdata.

The program ends by detaching ("closing") the HDF objects created above,
terminating the three interfaces initialized, and closing the HDF file.


Reading a vgroup
^^^^^^^^^^^^^^^^
The following program shows the contents of the vgroups contained inside
any HDF file::

    from pyhdf.HDF import *
    from pyhdf.V   import *
    from pyhdf.VS  import *
    from pyhdf.SD  import *

    import sys

    def describevg(refnum):

        # Describe the vgroup with the given refnum.

        # Open vgroup in read mode.
        vg = v.attach(refnum)
        print "----------------"
        print "name:", vg._name, "class:",vg._class, "tag,ref:",
        print vg._tag, vg._refnum

        # Show the number of members of each main object type.
        print "members: ", vg._nmembers,
        print "datasets:", vg.nrefs(HC.DFTAG_NDG),
        print "vdatas:  ", vg.nrefs(HC.DFTAG_VH),
        print "vgroups: ", vg.nrefs(HC.DFTAG_VG)

        # Read the contents of the vgroup.
        members = vg.tagrefs()

        # Display info about each member.
        index = -1
        for tag, ref in members:
            index += 1
            print "member index", index
            # Vdata tag
            if tag == HC.DFTAG_VH:
                vd = vs.attach(ref)
                nrecs, intmode, fields, size, name = vd.inquire()
                print "  vdata:",name, "tag,ref:",tag, ref
                print "    fields:",fields
                print "    nrecs:",nrecs
                vd.detach()

            # SDS tag
            elif tag == HC.DFTAG_NDG:
                sds = sd.select(sd.reftoindex(ref))
                name, rank, dims, type, nattrs = sds.info()
                print "  dataset:",name, "tag,ref:", tag, ref
                print "    dims:",dims
                print "    type:",type
                sds.endaccess()

            # VS tag
            elif tag == HC.DFTAG_VG:
                vg0 = v.attach(ref)
                print "  vgroup:", vg0._name, "tag,ref:", tag, ref
                vg0.detach()

            # Unhandled tag
            else:
                print "unhandled tag,ref",tag,ref

        # Close vgroup
        vg.detach()

    # Open HDF file in readonly mode.
    filename = sys.argv[1]
    hdf = HDF(filename)

    # Initialize the SD, V and VS interfaces on the file.
    sd = SD(filename)
    vs = hdf.vstart()
    v  = hdf.vgstart()

    # Scan all vgroups in the file.
    ref = -1
    while 1:
        try:
            ref = v.getid(ref)
        except HDF4Error,msg:    # no more vgroup
            break
        describevg(ref)

    # Terminate V, VS and SD interfaces.
    v.end()
    vs.end()
    sd.end()

    # Close HDF file.
    hdf.close()

The program starts by defining function describevg(), which is passed the
reference number of the vgroup to display. The function assumes that the
SD, VS and V interfaces have been previously initialized.

The function starts by attaching ("opening") the vgroup, and displaying
its name, class, tag and reference number. The number of members of the
three most important object types is then displayed, by calling the nrefs()
method with the predefined tags found inside the HDF.HC class.

The tagrefs() method is then called to get a list of all the vgroup members,
each member being identified by its tag and reference number. A 'for'
statement is entered to loop over each element of this list. The tag is
tested against the known values defined in the HDF.HC class: the outcome of
this test indicates how to process the member object.

A DFTAG_VH tag indicates we deal with a vdata. The vdata is attached, its
inquire() method called to display info about it, and the vdata is detached.
In the case of a DFTAG_NFG, we are facing a dataset. The dataset is
selected, info is obtained by calling the dataset info() method, and the
dataset is released. A DFTAG_VG indicates that the member is a vgroup. We
attach it, print its name, tag and reference number, then detach the
member vgroup. A warning is finally displayed if we hit upon a member of
an unknown type.

The function releases the vgroup just displayed and returns.

The main program starts by opening in readonly mode the HDF file passed
as argument on the command line. The SD, VS and V interfaces are
initialized, and the corresponding class instances are stored inside 'sd',
'vs' and 'v' global variables, respectively, for the use of the
describevg() function.

A while loop is then entered to access each vgroup in the file. A reference
number of -1 is passed on the first call to getid() to obtain the reference
number of the first vgroup. getid() returns a new reference number on each
subsequent call, and raises an exception when the last vgroup has been
retrieved. This exception is caught to break out of the loop, otherwise
describevg() is called to display the vgroup we have on hand.

Once the loop is over, the interfaces initialized before are terminated,
and the HDF file is closed.

You will notice that this program will display vgroups other than those
you have explicitly created. Those supplementary vgroups are created
by the HDF library for its own internal needs.

"""

import os, sys, types

from . import hdfext as _C

from .six.moves import xrange
from .HC import HC
from .VS import VD
from .error import HDF4Error, _checkErr

# List of names we want to be imported by an "from pyhdf.V import *"
# statement

__all__ = ['V', 'VG', 'VGAttr']

class V(object):
    """The V class implements the V (Vgroup) interface applied to an
    HDF file.
    To instantiate a V class, call the vgstart() method of an
    HDF instance. """

    def __init__(self, hinst):
        # Not to be called directly by the user.
        # A V object is instantiated using the vgstart()
        # method of an HDF instance.

        # Args:
        #    hinst    HDF instance
        # Returns:
        #    A V instance
        #
        # C library equivalent : Vstart (rather: Vinitialize)

        # Private attributes:
        #  _hdf_inst:       HDF instance

        # Note: Vstart is just a macro; use 'Vinitialize' instead
        # Note also thet the same C function is used to initialize the
        # VS interface.
        status = _C.Vinitialize(hinst._id)
        _checkErr('V', status, "cannot initialize V interface")
        self._hdf_inst = hinst


    def __del__(self):
        """Delete the instance, first calling the end() method
        if not already done.          """

        try:
            if self._hdf_inst:
                self.end()
        except:
            pass

    def end(self):
        """Close the V interface.

        Args::

          No argument

        Returns::

          None

        C library equivalent : Vend
                                                """

        # Note: Vend is just a macro; use 'Vfinish' instead
        # Note also the the same C function is used to end
        # the VS interface
        _checkErr('vend', _C.Vfinish(self._hdf_inst._id),
                  "cannot terminate V interface")
        self._hdf_inst = None

    def attach(self, num_name, write=0):
        """Open an existing vgroup given its name or its reference
        number, or create a new vgroup, returning a VG instance for
        that vgroup.

        Args::

          num_name      reference number or name of the vgroup to open,
                        or -1 to create a new vgroup; vcreate() can also
                        be called to create and name a new vgroup
          write         set to non-zero to open the vgroup in write mode
                        and to 0 to open it in readonly mode (default)

        Returns::

          VG instance for the vgroup

        An exception is raised if an attempt is made to open
        a non-existent vgroup.

        C library equivalent : Vattach
                                                """

        if isinstance(num_name, bytes):
            num = self.find(num_name)
        else:
            num = num_name
        vg_id = _C.Vattach(self._hdf_inst._id, num,
                           write and 'w' or 'r')
        _checkErr('vattach', vg_id, "cannot attach Vgroup")
        return VG(self, vg_id)

    def create(self, name):
        """Create a new vgroup, and assign it a name.

        Args::

          name   name to assign to the new vgroup

        Returns::

          VG instance for the new vgroup

        A create(name) call is equivalent to an attach(-1, 1) call,
        followed by a call to the setname(name) method of the instance.

        C library equivalent : no equivalent
                                                    """

        vg = self.attach(-1, 1)
        vg._name = name
        return vg

    def find(self, name):
        """Find a vgroup given its name, returning its reference
        number if found.

        Args::

          name     name of the vgroup to find

        Returns::

          vgroup reference number

        An exception is raised if the vgroup is not found.

        C library equivalent: Vfind
                                                  """

        refnum = _C.Vfind(self._hdf_inst._id, name)
        if not refnum:
            raise HDF4Error("vgroup not found")
        return refnum

    def findclass(self, name):
        """Find a vgroup given its class name, returning its reference
        number if found.

        Args::

          name     class name of the vgroup to find

        Returns::

          vgroup reference number

        An exception is raised if the vgroup is not found.

        C library equivalent: Vfind
                                                  """

        refnum = _C.Vfindclass(self._hdf_inst._id, name)
        if not refnum:
            raise HDF4Error("vgroup not found")
        return refnum

    def delete(self, num_name):
        """Delete from the HDF file the vgroup identified by its
        reference number or its name.

        Args::

          num_name    either the reference number or the name of
                      the vgroup to delete

        Returns::

          None

        C library equivalent : Vdelete
                                             """

        try:
            vg = self.attach(num_name, 1)
        except HDF4Error as msg:
            raise HDF4Error("delete: no such vgroup")

        # ATTENTION: The HDF documentation says that the vgroup_id
        #            is passed to Vdelete(). This is wrong.
        #            The vgroup reference number must instead be passed.
        refnum = vg._refnum
        vg.detach()
        _checkErr('delete', _C.Vdelete(self._hdf_inst._id, refnum),
                  "error deleting vgroup")

    def getid(self, ref):
        """Obtain the reference number of the vgroup following the
        vgroup with the given reference number .

        Args::

          ref    reference number of the vgroup after which to search;
                 set to -1 to start the search at the start of
                 the HDF file

        Returns::

          reference number of the vgroup past the one identified by 'ref'

        An exception is raised if the end of the vgroup is reached.

        C library equivalent : Vgetid
                                                    """

        num = _C.Vgetid(self._hdf_inst._id, ref)
        _checkErr('getid', num, "bad arguments or last vgroup reached")
        return num


class VG(object):
    """The VG class encapsulates the functionality of a vgroup.
    To instantiate a VG class, call the attach() or create() methods
    of a V class instance."""

    def __init__(self, vinst, id):
        # This constructor is not intended to be called directly
        # by the user program. The attach() method of an
        # V class instance should be called instead.

        # Arg:
        #  vinst       V instance from which the call is made
        #  id          vgroup identifier

        # Private attributes:
        #  _v_inst    V instance to which the vdata belongs
        #  _id        vgroup identifier

        self._v_inst = vinst
        self._id = id

    def __del__(self):
        """Delete the instance, first calling the detach() method
        if not already done.          """

        try:
            if self._id:
                self.detach()
        except:
            pass

    def __getattr__(self, name):
        """Some vgroup properties can be queried/set through the following
        attributes. Their names all start with an "_" to avoid
        clashes with user-defined attributes. Most are read-only.
        Only the _class and _name attributes can be modified.

        Name       RO  Description              C library routine
        -----      --  -----------------        -----------------
        _class         class name               Vgetclass/Vsetlass
        _name          name                     Vgetname/Vsetname
        _nattrs    X   number of attributes     Vnattrs
        _nmembers  X   number of vgroup members Vntagrefs
        _refnum    X   reference number         VQueryref
        _tag       X   tag                      VQuerytag
        _version   X   version number           Vgetversion

                                                         """

        # NOTE: python will call this method only if the attribute
        #       is not found in the object dictionary.

        # Check for a user defined attribute first.
        att = self.attr(name)
        if att._index is not None:   # Then the attribute exists
            return att.get()

        # Check for a predefined attribute
        if name == "_class":
            status, nm = _C.Vgetclass(self._id)
            _checkErr('_class', status, 'cannot get vgroup class')
            return nm

        elif name == "_name":
            status, nm = _C.Vgetname(self._id)
            _checkErr('_name', status, 'cannot get vgroup name')
            return nm

        elif name == "_nattrs":
            n = _C.Vnattrs(self._id)
            _checkErr('_nattrs', n, 'cannot get number of attributes')
            return n

        elif name == "_nmembers":
            n = _C.Vntagrefs(self._id)
            _checkErr('refnum', n, 'cannot get vgroup number of members')
            return n

        elif name == "_refnum":
            n = _C.VQueryref(self._id)
            _checkErr('refnum', n, 'cannot get vgroup reference number')
            return n

        elif name == "_tag":
            n = _C.VQuerytag(self._id)
            _checkErr('_tag', n, 'cannot get vgroup tag')
            return n

        elif name == "_version":
            n = _C.Vgetversion(self._id)
            _checkErr('_tag', n, 'cannot get vgroup version')
            return n

        else:
            raise AttributeError

    def __setattr__(self, name, value):

        # A name starting with an underscore will be treated as
        # a standard python attribute, and as an HDF attribute
        # otherwise.

        # Forbid assigning to readonly attributes
        if name in ["_nattrs", "_nmembers", "_refnum", "_tag", "_version"]:
            raise AttributeError("%s: read-only attribute" % name)

        # Read-write predefined attributes
        elif name == "_class":
            _checkErr(name, _C.Vsetclass(self._id, value),
                      'cannot set _class property')

        elif name == "_name":
            _checkErr(name, _C.Vsetname(self._id, value),
                      'cannot set _name property')

        # Try to set a user-defined attribute.
        else:
            _setattr(self, name, value)

    def insert(self, inst):
        """Insert a vdata or a vgroup in the vgroup.

        Args::

          inst  vdata or vgroup instance to add

        Returns::

          index of the inserted vdata or vgroup (0 based)

        C library equivalent : Vinsert
                                                  """

        if isinstance(inst, VD):
            id = inst._id
        elif isinstance(inst, VG):
            id = inst._id
        else:
            raise HDF4Error("insrt: bad argument")

        index = _C.Vinsert(self._id, id)
        _checkErr('insert', index, "cannot insert in vgroup")
        return index

    def add(self, tag, ref):
        """Add to the vgroup an object identified by its tag and
        reference number.

        Args::

          tag       tag of the object to add
          ref       reference number of the object to add

        Returns::

          total number of objects in the vgroup after the addition

        C library equivalent : Vaddtagref
                                              """

        n = _C.Vaddtagref(self._id, tag, ref)
        _checkErr('addtagref', n, 'invalid arguments')
        return n

    def delete(self, tag, ref):
        """Delete from the vgroup the member identified by its tag
        and reference number.

        Args::

          tag    tag of the member to delete
          ref    reference number of the member to delete

        Returns::

          None

        Only the link of the member with the vgroup is deleted.
        The member object is not deleted.

        C library equivalent : Vdeletatagref
                                                  """

        _checkErr('delete', _C.Vdeletetagref(self._id, tag, ref),
                  "error deleting member")

    def detach(self):
        """Terminate access to the vgroup.

        Args::

          no argument

        Returns::

          None

        C library equivalent : Vdetach
                                              """

        _checkErr('detach', _C.Vdetach(self._id),
                  "cannot detach vgroup")
        self._id = None

    def tagref(self, index):
        """Get the tag and reference number of a vgroup member,
        given the index number of that member.

        Args::

          index   member index (0 based)

        Returns::

          2-element tuple:
            - member tag
            - member reference number

        C library equivalent : Vgettagref
                                                  """

        status, tag, ref = _C.Vgettagref(self._id, index)
        _checkErr('tagref', status, "illegal arguments")
        return tag, ref

    def tagrefs(self):
        """Get the tags and reference numbers of all the vgroup
        members.

        Args::

          no argument

        Returns::

          list of (tag,ref) tuples, one for each vgroup member

        C library equivalent : Vgettagrefs
                                                      """
        n = self._nmembers
        ret = []
        if n:
            tags = _C.array_int32(n)
            refs = _C.array_int32(n)
            k = _C.Vgettagrefs(self._id, tags, refs, n)
            _checkErr('tagrefs', k, "error getting tags and refs")
            for m in xrange(k):
                ret.append((tags[m], refs[m]))

        return ret

    def inqtagref(self, tag, ref):
        """Determines if an object identified by its tag and reference
        number belongs to the vgroup.

        Args::

          tag      tag of the object to check
          ref      reference number of the object to check

        Returns::

          False (0) if the object does not belong to the vgroup,
          True  (1) otherwise

        C library equivalent : Vinqtagref
                                             """

        return _C.Vinqtagref(self._id, tag, ref)

    def nrefs(self, tag):
        """Determine the number of tags of a given type in a vgroup.

        Args::

          tag    tag type to look for in the vgroup

        Returns::

          number of members identified by this tag type

        C library equivalent : Vnrefs
                                              """

        n = _C.Vnrefs(self._id, tag)
        _checkErr('nrefs', n, "bad arguments")
        return n

    def isvg(self, ref):
        """Determines if the member of a vgoup is a vgroup.

        Args::

          ref      reference number of the member to check

        Returns::

          False (0) if the member is not a vgroup
          True  (1) otherwise

        C library equivalent : Visvg
                                             """

        return _C.Visvg(self._id, ref)

    def isvs(self, ref):
        """Determines if the member of a vgoup is a vdata.

        Args::

          ref      reference number of the member to check

        Returns::

          False (0) if the member is not a vdata,
          True  (1) otherwise

        C library equivalent : Visvs
                                             """

        return _C.Visvs(self._id, ref)

    def attr(self, name_index):
        """Create a VGAttr instance representing a vgroup attribute.

        Args::

          name_index  attribute name or attribute index number; if a
                      name is given the attribute may not exist; in that
                      case, it will be created when the VGAttr
                      instance set() method is called

        Returns::

          VGAttr instance for the attribute. Call the methods of this
          class to query, read or set the attribute.

        C library equivalent : no equivalent

                                """

        return VGAttr(self, name_index)

    def attrinfo(self):
        """Return info about all the vgroup attributes.

        Args::

          no argument

        Returns::

          dictionary describing each vgroup attribute; for each attribute,
          a (name,data) pair is added to the dictionary, where 'data' is
          a tuple holding:

          - attribute data type (one of HC.xxx constants)
          - attribute order
          - attribute value
          - attribute size in bytes

        C library equivalent : no equivalent
                                                  """

        dic = {}
        for n in range(self._nattrs):
            att = self.attr(n)
            name, type, order, size = att.info()
            dic[name] = (type, order, att.get(), size)

        return dic


    def findattr(self, name):
        """Search the vgroup for a given attribute.

        Args::

          name    attribute name

        Returns::

          if found, VGAttr instance describing the attribute
          None otherwise

         C library equivalent : Vfindattr
                                                  """

        try:
            att = self.attr(name)
            if att._index is None:
                att = None
        except HDF4Error:
            att = None
        return att


class VGAttr(object):
    """The VGAttr class encapsulates methods used to set and query
    attributes defined on a vgroup. To create an instance of this class,
    call the attr() method of a VG class.    """

    def __init__(self, obj, name_or_index):
        # This constructor should not be called directly by the user
        # program. The attr() method of a VG class must be called to
        # instantiate this class.

        # Args:
        #  obj            VG instance to which the attribute belongs
        #  name_or_index  name or index of the attribute; if a name is
        #                 given, an attribute with that name will be
        #                 searched, if not found, a new index number will
        #                 be generated

        # Private attributes:
        #  _v_inst        V instance
        #  _index         attribute index or None
        #  _name          attribute name or None

        self._v_inst = obj
        # Name is given. Attribute may exist or not.
        if isinstance(name_or_index, type('')):
            self._name = name_or_index
            self._index = _C.Vfindattr(self._v_inst._id, self._name)
            if self._index < 0:
                self._index = None
        # Index is given. Attribute must exist.
        else:
            self._index = name_or_index
            status, self._name, data_type, n_values, size = \
                    _C.Vattrinfo(self._v_inst._id, self._index)
            _checkErr('attr', status, 'non-existent attribute')

    def get(self):
        """Retrieve the attribute value.

        Args::

          no argument

        Returns::

          attribute value(s); a list is returned if the attribute
          is made up of more than one value, except in the case of a
          string-valued attribute (data type HC.CHAR8) where the
          values are returned as a string

        Note that a vgroup attribute can also be queried like a standard
        python class attribute by applying the usual "dot notation" to a
        VG instance.

        C library equivalent : Vgetattr

                                                """
        # Make sure the attribute exists.
        if self._index is None:
            raise HDF4Error("non existent attribute")
        # Obtain attribute type and the number of values.
        status, aName, data_type, n_values, size = \
                    _C.Vattrinfo(self._v_inst._id, self._index)
        _checkErr('get', status, 'illegal parameters')

        # Get attribute value.
        convert = _array_to_ret
        if data_type == HC.CHAR8:
            buf = _C.array_byte(n_values)
            convert = _array_to_str

        elif data_type in [HC.UCHAR8, HC.UINT8]:
            buf = _C.array_byte(n_values)

        elif data_type == HC.INT8:
            buf = _C.array_int8(n_values)

        elif data_type == HC.INT16:
            buf = _C.array_int16(n_values)

        elif data_type == HC.UINT16:
            buf = _C.array_uint16(n_values)

        elif data_type == HC.INT32:
            buf = _C.array_int32(n_values)

        elif data_type == HC.UINT32:
            buf = _C.array_uint32(n_values)

        elif data_type == HC.FLOAT32:
            buf = _C.array_float32(n_values)

        elif data_type == HC.FLOAT64:
            buf = _C.array_float64(n_values)

        else:
            raise HDF4Error("get: attribute index %d has an "\
                             "illegal or unupported type %d" % \
                             (self._index, data_type))

        status = _C.Vgetattr(self._v_inst._id, self._index, buf)
        _checkErr('get', status, 'illegal attribute ')
        return convert(buf, n_values)

    def set(self, data_type, values):
        """Set the attribute value.

        Args::

          data_type    : attribute data type (see constants HC.xxx)
          values       : attribute value(s); specify a list to create
                         a multi-valued attribute; a string valued
                         attribute can be created by setting 'data_type'
                         to HC.CHAR8 and 'values' to the corresponding
                         string

                         If the attribute already exists, it will be
                         updated. However, it is illegal to try to change
                         its data type or its order (number of values).

        Returns::

          None

        Note that a vgroup attribute can also be set like a standard
        python class attribute by applying the usual "dot notation" to a
        VG instance.

        C library equivalent : Vsetattr

                                                  """
        try:
            n_values = len(values)
        except:
            values = [values]
            n_values = 1
        if data_type == HC.CHAR8:
            buf = _C.array_byte(n_values)
            # Allow values to be passed as a string.
            # Noop if a list is passed.
            values = list(values)
            for n in range(n_values):
                values[n] = ord(values[n])

        elif data_type in [HC.UCHAR8, HC.UINT8]:
            buf = _C.array_byte(n_values)

        elif data_type == HC.INT8:
            # SWIG refuses negative values here. We found that if we
            # pass them as byte values, it will work.
            buf = _C.array_int8(n_values)
            values = list(values)
            for n in range(n_values):
                v = values[n]
                if v >= 0:
                    v &= 0x7f
                else:
                    v = abs(v) & 0x7f
                    if v:
                        v = 256 - v
                    else:
                        v = 128         # -128 in 2s complement
                values[n] = v

        elif data_type == HC.INT16:
            buf = _C.array_int16(n_values)

        elif data_type == HC.UINT16:
            buf = _C.array_uint16(n_values)

        elif data_type == HC.INT32:
            buf = _C.array_int32(n_values)

        elif data_type == HC.UINT32:
            buf = _C.array_uint32(n_values)

        elif data_type == HC.FLOAT32:
            buf = _C.array_float32(n_values)

        elif data_type == HC.FLOAT64:
            buf = _C.array_float64(n_values)

        else:
            raise HDF4Error("set: illegal or unimplemented data_type")

        for n in range(n_values):
            buf[n] = values[n]
        status = _C.Vsetattr(self._v_inst._id, self._name, data_type,
                             n_values, buf)
        _checkErr('set', status, 'cannot execute')
        # Update the attribute index
        self._index = _C.Vfindattr(self._v_inst._id, self._name)
        if self._index < 0:
            raise HDF4Error("set: error retrieving attribute index")

    def info(self):
        """Retrieve info about the attribute.

        Args::

          no argument

        Returns::

          4-element tuple with the following components:
            -attribute name
            -attribute data type (one of HC.xxx constants)
            -attribute order (number of values)
            -attribute size in bytes

        C library equivalent : Vattrinfo
                                                           """

        # Make sure the attribute exists.
        if self._index is None:
            raise HDF4Error("non existent attribute")

        status, name, type, order, size = \
                _C.Vattrinfo(self._v_inst._id, self._index)
        _checkErr('info', status, "execution error")
        return name, type, order, size


###########################
# Support functions
###########################

def _setattr(obj, name, value):
    # Called by the __setattr__ method of the VG object.
    #
    #  obj   instance on which the attribute is set
    #  name  attribute name
    #  value attribute value

    # Treat a name starting with and underscore as that of a
    # standard python instance attribute.
    if name[0] == '_':
        obj.__dict__[name] = value
        return

    # Treat everything else as an HDF attribute.
    if type(value) not in [list, tuple]:
        value = [value]
    typeList = []
    for v in value:
        t = type(v)
        # Prohibit mixing numeric types and strings.
        if t in [int, float] and \
               not bytes in typeList:
            if t not in typeList:
                typeList.append(t)
        # Prohibit sequence of strings or a mix of numbers and string.
        elif t == bytes and not typeList:
            typeList.append(t)
        else:
            typeList = []
            break
    if bytes in typeList:
        xtype = HC.CHAR8
        value = value[0]
    # double is "stronger" than int
    elif float in typeList:
        xtype = HC.FLOAT64
    elif int in typeList:
        xtype = HC.INT32
    else:
        raise HDF4Error("Illegal attribute value")

    # Assign value
    try:
        a = obj.attr(name)
        a.set(xtype, value)
    except HDF4Error as msg:
        raise HDF4Error("cannot set attribute: %s" % msg)



def _array_to_ret(buf, nValues):

    # Convert array 'buf' to a scalar or a list.

    if nValues == 1:
        ret = buf[0]
    else:
        ret = []
        for i in xrange(nValues):
            ret.append(buf[i])
    return ret

def _array_to_str(buf, nValues):

    # Convert array of bytes 'buf' to a string.

    # Return empty string if there is no value.
    if nValues == 0:
        return ""
    # When there is just one value, _array_to_ret returns a scalar
    # over which we cannot iterate.
    if nValues == 1:
        chrs = [chr(buf[0])]
    else:
        chrs = [chr(b) for b in _array_to_ret(buf, nValues)]
    # Strip NULL at end
    if chrs[-1] == '\0':
        del chrs[-1]
    return ''.join(chrs)



#!/usr/bin/env python
"""
This script runs all the examples in examples directory.
It should be run from the top-level directory (containing setup.py).
"""

import glob
import os
import sys
import subprocess

try:
    from subprocess import DEVNULL # py3k
except ImportError:
    DEVNULL = open(os.devnull, 'wb')

class ExampleTester:
    failed = []
    passed = 0
    rootdir =  os.getcwd()

    def __init__(self):
        subprocess.check_call([sys.executable, "setup.py", "build_ext", "--inplace"])
        os.environ["PYTHONPATH"] = self.rootdir

    def run(self, d, cmd, **kwargs):
        """
        Run command cmd in directory d.
        """
        print("running %s in %s ..." % (cmd, d))
        os.chdir(os.path.join(self.rootdir, d))
        r = subprocess.call([sys.executable] + cmd, **kwargs)
        if r != 0:
            self.failed.append((d, cmd, r))
        else:
            self.passed += 1
        os.chdir(self.rootdir)
        return r

    def report(self):
        print(self.passed, "example(s) ran successfully")
        if len(self.failed) > 0:
            print("failed examples:")
            for ex in self.failed:
                print("  %s in %s failed with exit code %s" % (ex[1], ex[0], ex[2]))
            sys.exit(2)
        sys.exit(0)

def main():
    t = ExampleTester()
    t.run("examples/compress", ["test-compress.py"], stdout=DEVNULL)
    t.run("examples/vgroup", ["vgwrite.py"])
    t.run("examples/vgroup", ["vgread.py", "inventory.hdf"], stdout=DEVNULL)
    t.run("examples/inventory", ["inventory_1-1.py"])
    t.run("examples/inventory", ["inventory_1-2.py"])
    t.run("examples/inventory", ["inventory_1-3.py"])
    t.run("examples/inventory", ["inventory_1-4.py"], stdout=DEVNULL)
    t.run("examples/inventory", ["inventory_1-5.py"], stdout=DEVNULL)
    t.run("examples/txttohdf", ["txttohdf.py"])

    # These HDF files were generated by above examples
    for g in sorted(glob.glob("examples/*/*.hdf")):
        hdffile = os.path.join("../..", g)
        t.run("examples/hdfstruct", ["hdfstruct.py", hdffile], stdout=DEVNULL)

    t.report()

if __name__ == '__main__':
    main()



from pyhdf.HDF import *
from pyhdf.VS import *

f = HDF('inventory.hdf',         # Open file 'inventory.hdf'
        HC.WRITE|HC.CREATE)      # creating it if it does not exist
vs = f.vstart()                  # init vdata interface
vd = vs.attach('INVENTORY', 1)   # attach vdata 'INVENTORY' in write mode

# Update the `status' vdata attribute. The attribute length must not
# change. We call the attribute info() method, which returns a list where
# number of values (eg string length) is stored at index 2.
# We then assign a left justified string of exactly that length.
len = vd.attr('status').info()[2]
vd.status = '%-*s' % (len, 'phase 2 done')

vd[vd._nrecs:] =     (                     # append 2 records
          ('A4321', 'axe', 5, 1.5, 25),    # first record
          ('C3214', 'cup', 100, 0.1, 3.25) # second record
                    )
vd.detach()               # "close" the vdata

vs.end()                  # terminate the vdata interface
f.close()                 # close the HDF file



from __future__ import print_function

from pyhdf.HDF import *
from pyhdf.V   import *
from pyhdf.VS  import *
from pyhdf.SD  import *

import sys

def describevg(refnum):

    # Describe the vgroup with the given refnum.

    # Open vgroup in read mode.
    vg = v.attach(refnum)
    print("----------------")
    print("name:", vg._name, "class:",vg._class, "tag,ref:",vg._tag, vg._refnum)

    # Show the number of members of each main object type.
    print("# members:  ", vg._nmembers,\
           "# datasets:", vg.nrefs(HC.DFTAG_NDG),\
           "# vdatas:  ", vg.nrefs(HC.DFTAG_VH),\
           "# vgroups: ", vg.nrefs(HC.DFTAG_VG))

    # Read the contents of the vgroup.
    members = vg.tagrefs()

    # Display info about each member.
    index = -1
    for tag, ref in members:
        index += 1
        print("member index", index)
        # Vdata tag
        if tag == HC.DFTAG_VH:
            vd = vs.attach(ref)
            nrecs, intmode, fields, size, name = vd.inquire()
            print("  vdata:",name, "tag,ref:",tag, ref)
            print("    fields:",fields)
            print("    nrecs:",nrecs)
            vd.detach()

        # SDS tag
        elif tag == HC.DFTAG_NDG:
            sds = sd.select(sd.reftoindex(ref))
            name, rank, dims, type, nattrs = sds.info()
            print("  dataset:",name, "tag,ref:", tag, ref)
            print("    dims:",dims)
            print("    type:",type)
            sds.endaccess()

        # VS tag
        elif tag == HC.DFTAG_VG:
            vg0 = v.attach(ref)
            print("  vgroup:", vg0._name, "tag,ref:", tag, ref)
            vg0.detach()

        # Unhandled tag
        else:
            print("unhandled tag,ref",tag,ref)

    # Close vgroup
    vg.detach()

# Open HDF file in readonly mode.
filename = sys.argv[1]
hdf = HDF(filename)

# Initialize the SD, V and VS interfaces on the file.
sd = SD(filename)
vs = hdf.vstart()
v  = hdf.vgstart()

# Scan all vgroups in the file.
ref = -1
while True:
    try:
        ref = v.getid(ref)
    except HDF4Error as msg:    # no more vgroup
        break
    describevg(ref)

# Terminate V, VS and SD interfaces.
v.end()
vs.end()
sd.end()

# Close HDF file.
hdf.close()



#!/usr/bin/env python
"""pyhdf: Python interface to the NCSA HDF4 library.

The pyhdf package wraps the functionality of the NCSA HDF version
4 library inside a Python OOP framework. The SD (scientific dataset),
VS (Vdata) and V (Vgroup) APIs are currently implemented.  SD datasets
are read/written through numpy arrays. NetCDF files can also be read
and modified with pyhdf.
"""

from __future__ import print_function

DOCLINES = __doc__.split("\n")

# Allows bdist_egg to work if you have setuptools installed.
# This import must be before the numpy.distutils import of setup.
#  Otherwise, no harm.
try:
    import setuptools
except:
    pass

from numpy.distutils.core import setup, Extension

import sys
import os
import os.path as path
import shlex

CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved
Programming Language :: C
Programming Language :: Python
Programming Language :: Python :: 3
Topic :: Software Development
Topic :: Scientific/Engineering
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

def _find_args(pat, env):
    val = os.environ.get(env, [])
    if val:
        val = val.split(os.pathsep)
    try:
        k = sys.argv.index(pat)
        val.extend(sys.argv[k+1].split(os.pathsep))
        del sys.argv[k]
        del sys.argv[k]
    except ValueError:
        pass
    return val

# A Debian based linux distribution might be using libhdf4 (contains netcdf
# routines) or libhdf4-alt (does not contain netcdf routines). This function
# tries to detect if the alt version should be used.
def _use_hdf4alt(libdirs):
    if not sys.platform.startswith("linux"):
        return False
    libdirs.extend(os.environ.get("LD_LIBRARY_PATH", "").split(os.pathsep))
    libdirs.append("/usr/lib")
    libdirs.append("/usr/local/lib")
    libdirs.append("/lib")
    for d in libdirs:
        if os.path.exists(os.path.join(d, "libdfalt.so")) and \
           os.path.exists(os.path.join(d, "libmfhdfalt.so")):
            return True
    return False

include_dirs = _find_args('-i', 'INCLUDE_DIRS')
library_dirs = _find_args('-l', 'LIBRARY_DIRS')
szip_installed = 'SZIP' in os.environ
compress = 'NO_COMPRESS' not in os.environ
extra_link_args = None
if "LINK_ARGS" in os.environ:
    extra_link_args = shlex.split(os.environ["LINK_ARGS"])


msg = 'Cannot proceed without the HDF4 library.  Please ' \
      'export INCLUDE_DIRS and LIBRARY_DIRS as explained' \
      'in the INSTALL file.'

if sys.platform.startswith('linux'):
    # libhdf4 header files on most linux distributations
    # (e.g. Debian/Ubuntu, CentOS) are stored in /usr/include/hdf
    d = "/usr/include/hdf/"
    if not include_dirs and os.path.exists(d):
        include_dirs.append(d)

for p in include_dirs + library_dirs:
    if not path.exists(p):
        print("\n******\n%s not found\n******\n\n" % p)
        raise RuntimeError(msg)

if sys.platform == 'win32':
    # Find DLL path
    dll_path = ''
    for p in library_dirs:
        if path.exists(p + os.path.sep + "mfhdf.dll"):
            dll_path = p + os.path.sep
            break
    if dll_path == '':
        print("library_dirs =", library_dirs)
        raise RuntimeError("Cannot find required HDF4 DLLs -- check LIBRARY_DIRS")

if sys.platform == 'win32':
    libraries = ["mfhdf", "hdf", "xdr" ]
elif _use_hdf4alt(library_dirs):
    libraries = ["mfhdfalt", "dfalt"]
else:
    libraries = ["mfhdf", "df"]

if szip_installed:
    extra_compile_args = []
    if sys.platform == 'win32':
        libraries += ["szlib"]
    else:
        libraries += ["sz"]
else:
    extra_compile_args = ["-DNOSZIP"]
if sys.platform == 'win32':
    libraries += ["libjpeg", "zlib", "ws2_32"]
else:
    libraries += ["jpeg", "z"]

if not compress:
    extra_compile_args += ["-DNOCOMPRESS"]

_hdfext = Extension('pyhdf._hdfext',
                    sources      = ["pyhdf/hdfext_wrap.c"],
                    include_dirs = include_dirs,
                    extra_compile_args = extra_compile_args,
                    library_dirs = library_dirs,
                    extra_link_args=extra_link_args,
                    libraries = libraries,
                    )

if sys.platform == 'win32':
    data_files = [("pyhdf", [dll_path + x for x in ["mfhdf.dll", "hdf.dll"]])]
else:
    data_files = []

setup(name         = 'pyhdf',
      maintainer       = 'pyhdf authors',
      author       = 'Andre Gosselin et al.',
      description  = DOCLINES[0],
      keywords     = ['hdf4', 'netcdf', 'numpy', 'python', 'pyhdf'],
      license      = 'MIT',
      long_description = "\n".join(DOCLINES[2:]),
      url          = 'https://github.com/fhs/pyhdf',
      version      = '0.10.5',
      packages     = ['pyhdf'],
      ext_modules  = [_hdfext],
      data_files   = data_files,
      provides     = ['pyhdf'],
      classifiers  = [_f for _f in CLASSIFIERS.split('\n') if _f],
      platforms    = ["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
      )



# $Id: HDF.py,v 1.3 2005-07-14 01:36:41 gosselin_a Exp $
# $Log: not supported by cvs2svn $
# Revision 1.2  2004/08/02 15:36:04  gosselin
# pyhdf-0.7-1
#
# Revision 1.1  2004/08/02 15:22:59  gosselin
# Initial revision
#
# Author: Andre Gosselin
#         Maurice-Lamontagne Institute
#         gosselina@dfo-mpo.gc.ca

"""
Basic API (:mod:`pyhdf.HDF`)
============================

A module of the pyhdf package implementing the basic API of the
NCSA HDF4 library.

Introduction
------------
The role of the HDF module is to provide support to other modules of the
pyhdf package. It defines constants specifying file opening modes and
various data types, methods for accessing files, plus a few utility
functions to query library version and check if a file is an HDF one.

It should be noted that, among the modules of the pyhdf package, SD is
special in the sense that it is self-contained and does not need support
from the HDF module. For example, SD provides its own file opening and
closing methods, whereas VS uses methods of the HDF.HDF class for that.

Functions and classes summary
-----------------------------
The HDF module provides the following classes.

  HC
      The HC class holds constants defining opening modes and
      various data types.

  HDF
      The HDF class provides methods to open and close an HDF file,
      and return instances of the major HDF APIs (except SD).

      To instantiate an HDF class, call the HDF() constructor.

      methods:
        constructors:
          HDF()     open an HDF file, creating the file if necessary,
                    and return an HDF instance
          vstart()  initialize the VS (Vdata) API over the HDF file and
                    return a VS instance
          vgstart() initialize the V (Vgroup) interface over the HDF file
                    and return a V instance.


        closing file
          close()  close the HDF file

        inquiry
          getfileversion()  return info about the version of the  HDF file

The HDF module also offers the following functions.

  inquiry
    getlibversion()    return info about the version of the library
    ishdf()            determine whether a file is an HDF file


"""

import os, sys, types

from . import hdfext as _C
from .six.moves import xrange
from .HC import HC

# NOTE: The vstart() and vgstart() modules need to access the
#       VS and V modules, resp. We could simply import those
#       two modules, but then they would always be loaded and this
#       may not be what the user wants. Instead of forcing the
#       systematic import, we import the package `pyhdf',
#       and access the needed constructors by writing
#       'pyhdf.VS.VS()' and 'pyhdf.V.V()'. Until the VS or
#       V modules are imported, those statements will give an
#       error (undefined attribute). Once the user has imported
#       the modules, the error will disappear.

import pyhdf

from .error import HDF4Error, _checkErr

# List of names we want to be imported by an "from pyhdf.HDF import *"
# statement

__all__ = ['HDF', 'HDF4Error',
           'HC',
           'getlibversion', 'ishdf']

def getlibversion():
    """Get the library version info.

    Args:
      no argument
    Returns:
      4-element tuple with the following components:
        -major version number (int)
        -minor version number (int)
        -complete library version number (int)
        -additional information (string)

    C library equivalent : Hgetlibversion
                                                   """

    status, major_v, minor_v, release, info = _C.Hgetlibversion()
    _checkErr('getlibversion', status, "cannot get lib version")
    return major_v, minor_v, release, info

def ishdf(filename):
    """Determine whether a file is an HDF file.

    Args:
      filename  name of the file to check
    Returns:
      1 if the file is an HDF file, 0 otherwise

    C library equivalent : Hishdf
                                            """

    return _C.Hishdf(filename)


class HDF(object):
    """The HDF class encapsulates the basic HDF functions.
    Its main use is to open and close an HDF file, and return
    instances of the major HDF APIs (except for SD).
    To instantiate an HDF class, call the HDF() constructor. """

    def __init__(self, path, mode=HC.READ, nblocks=0):
        """HDF constructor: open an HDF file, creating the file if
        necessary.

        Args:
          path    name of the HDF file to open
          mode    file opening mode; this mode is a set of binary flags
                  which can be ored together

                      HC.CREATE   combined with HC.WRITE to create file
                                  if it does not exist
                      HC.READ     open file in read-only access (default)
                      HC.TRUNC    if combined with HC.WRITE, overwrite
                                  file if it already exists
                      HC.WRITE    open file in read-write mode; if file
                                  exists it is updated, unless HC.TRUNC is
                                  set, in which case it is erased and
                                  recreated; if file does not exist, an
                                  error is raised unless HC.CREATE is set,
                                  in which case the file is created

                   Note an important difference in the way CREATE is
                   handled by the HDF C library and the pyhdf package.
                   In the C library, CREATE indicates that a new file should
                   always be created, overwriting an existing one if
                   any. For pyhdf, CREATE indicates a new file should be
                   created only if it does not exist, and the overwriting
                   of an already existing file must be explicitly asked
                   for by setting the TRUNC flag.

                   Those differences were introduced so as to harmonize
                   the way files are opened in the pycdf and pyhdf
                   packages. Also, this solves a limitation in the
                   hdf (and netCDF) library, where there is no easy way
                   to implement the frequent requirement that an existent
                   file be opened in read-write mode, or created
                   if it does not exist.

          nblocks  number of data descriptor blocks in a block with which
                   to create the file; the parameter is ignored if the file
                   is not created; 0 asks to use the default

        Returns:
          an HDF instance

        C library equivalent : Hopen
                                                     """
        # Private attributes:
        #  _id:       file id (NOTE: not compatile with the SD file id)

        # See if file exists.
        exists = os.path.exists(path)

        if HC.WRITE & mode:
            if exists:
                if HC.TRUNC & mode:
                    try:
                        os.remove(path)
                    except Exception as msg:
                        raise HDF4Error(msg)
                    mode = HC.CREATE
                else:
                    mode = HC.WRITE
            else:
                if HC.CREATE & mode:
                    mode = HC.CREATE
                else:
                    raise HDF4Error("HDF: no such file")
        else:
            if exists:
                if mode & HC.READ:
                    mode = HC.READ     # clean mode
                else:
                    raise HDF4Error("HDF: invalid mode")
            else:
                raise HDF4Error("HDF: no such file")

        id = _C.Hopen(path, mode, nblocks)
        _checkErr('HDF', id, "cannot open %s" % path)
        self._id = id


    def __del__(self):
        """Delete the instance, first calling the end() method
        if not already done.          """

        try:
            if self._id:
                self.close()
        except:
            pass

    def close(self):
        """Close the HDF file.

        Args:
          no argument
        Returns:
          None

        C library equivalent : Hclose
                                                """

        _checkErr('close', _C.Hclose(self._id), "cannot close file")
        self._id = None

    def getfileversion(self):
        """Get file version info.

        Args:
          no argument
        Returns:
          4-element tuple with the following components:
            -major version number (int)
            -minor version number (int)
            -complete library version number (int)
            -additional information (string)

        C library equivalent : Hgetlibversion
                                                   """

        status, major_v, minor_v, release, info = _C.Hgetfileversion(self._id)
        _checkErr('getfileversion', status, "cannot get file version")
        return major_v, minor_v, release, info

    def vstart(self):
        """Initialize the VS API over the file and return a VS instance.

        Args:
          no argument
        Returns:
          VS instance

        C library equivalent : Vstart (in fact: Vinitialize)
                                                              """
        # See note at top of file.
        return pyhdf.VS.VS(self)

    def vgstart(self):
        """Initialize the V API over the file and return a V instance.

        Args:
          no argument
        Returns:
          V instance

        C library equivalent : Vstart (in fact: Vinitialize)
                                                              """
        # See note at top of file.
        return pyhdf.V.V(self)



###########################
# Support functions
###########################


def _array_to_ret(buf, nValues):

    # Convert array 'buf' to a scalar or a list.

    if nValues == 1:
        ret = buf[0]
    else:
        ret = []
        for i in xrange(nValues):
            ret.append(buf[i])
    return ret

def _array_to_str(buf, nValues):

    # Convert array of bytes 'buf' to a string.

    # Return empty string if there is no value.
    if nValues == 0:
        return ""
    # When there is just one value, _array_to_ret returns a scalar
    # over which we cannot iterate.
    if nValues == 1:
        chrs = [chr(buf[0])]
    else:
        chrs = [chr(b) for b in _array_to_ret(buf, nValues)]
    # Strip NULL at end
    if chrs[-1] == '\0':
        del chrs[-1]
    return ''.join(chrs)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_hdfext')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_hdfext')
    _hdfext = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_hdfext', [dirname(__file__)])
        except ImportError:
            import _hdfext
            return _hdfext
        try:
            _mod = imp.load_module('_hdfext', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _hdfext = swig_import_helper()
    del swig_import_helper
else:
    import _hdfext
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

DFNT_NONE = _hdfext.DFNT_NONE
DFNT_QUERY = _hdfext.DFNT_QUERY
DFNT_VERSION = _hdfext.DFNT_VERSION
DFNT_FLOAT32 = _hdfext.DFNT_FLOAT32
DFNT_FLOAT = _hdfext.DFNT_FLOAT
DFNT_FLOAT64 = _hdfext.DFNT_FLOAT64
DFNT_DOUBLE = _hdfext.DFNT_DOUBLE
DFNT_FLOAT128 = _hdfext.DFNT_FLOAT128
DFNT_INT8 = _hdfext.DFNT_INT8
DFNT_UINT8 = _hdfext.DFNT_UINT8
DFNT_INT16 = _hdfext.DFNT_INT16
DFNT_UINT16 = _hdfext.DFNT_UINT16
DFNT_INT32 = _hdfext.DFNT_INT32
DFNT_UINT32 = _hdfext.DFNT_UINT32
DFNT_INT64 = _hdfext.DFNT_INT64
DFNT_UINT64 = _hdfext.DFNT_UINT64
DFNT_INT128 = _hdfext.DFNT_INT128
DFNT_UINT128 = _hdfext.DFNT_UINT128
DFNT_UCHAR8 = _hdfext.DFNT_UCHAR8
DFNT_UCHAR = _hdfext.DFNT_UCHAR
DFNT_CHAR8 = _hdfext.DFNT_CHAR8
DFNT_CHAR = _hdfext.DFNT_CHAR
DFNT_CHAR16 = _hdfext.DFNT_CHAR16
DFNT_UCHAR16 = _hdfext.DFNT_UCHAR16
SD_UNLIMITED = _hdfext.SD_UNLIMITED
SD_FILL = _hdfext.SD_FILL
SD_NOFILL = _hdfext.SD_NOFILL
CHAR_BUFFER_SIZE = _hdfext.CHAR_BUFFER_SIZE
ATTRIB_BUFFER_SIZE = _hdfext.ATTRIB_BUFFER_SIZE
DFACC_READ = _hdfext.DFACC_READ
DFACC_WRITE = _hdfext.DFACC_WRITE
DFACC_CREATE = _hdfext.DFACC_CREATE
DFACC_ALL = _hdfext.DFACC_ALL
DFACC_RDONLY = _hdfext.DFACC_RDONLY
DFACC_RDWR = _hdfext.DFACC_RDWR
DFACC_CLOBBER = _hdfext.DFACC_CLOBBER
DFACC_BUFFER = _hdfext.DFACC_BUFFER
DFACC_APPENDABLE = _hdfext.DFACC_APPENDABLE
DFACC_CURRENT = _hdfext.DFACC_CURRENT
DFACC_OLD = _hdfext.DFACC_OLD
COMP_CODE_NONE = _hdfext.COMP_CODE_NONE
COMP_CODE_RLE = _hdfext.COMP_CODE_RLE
COMP_CODE_NBIT = _hdfext.COMP_CODE_NBIT
COMP_CODE_SKPHUFF = _hdfext.COMP_CODE_SKPHUFF
COMP_CODE_DEFLATE = _hdfext.COMP_CODE_DEFLATE
COMP_CODE_SZIP = _hdfext.COMP_CODE_SZIP
DFTAG_NDG = _hdfext.DFTAG_NDG
DFTAG_VH = _hdfext.DFTAG_VH
DFTAG_VG = _hdfext.DFTAG_VG
H4_MAX_VAR_DIMS = _hdfext.H4_MAX_VAR_DIMS
class array_byte(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_byte, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_byte, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_byte(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_byte
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_byte___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_byte___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_byte_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_byte_frompointer)
    else:
        frompointer = _hdfext.array_byte_frompointer
array_byte_swigregister = _hdfext.array_byte_swigregister
array_byte_swigregister(array_byte)

def array_byte_frompointer(t):
    return _hdfext.array_byte_frompointer(t)
array_byte_frompointer = _hdfext.array_byte_frompointer

class array_int8(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_int8, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_int8, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_int8(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_int8
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_int8___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_int8___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_int8_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_int8_frompointer)
    else:
        frompointer = _hdfext.array_int8_frompointer
array_int8_swigregister = _hdfext.array_int8_swigregister
array_int8_swigregister(array_int8)

def array_int8_frompointer(t):
    return _hdfext.array_int8_frompointer(t)
array_int8_frompointer = _hdfext.array_int8_frompointer

class array_int16(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_int16, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_int16, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_int16(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_int16
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_int16___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_int16___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_int16_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_int16_frompointer)
    else:
        frompointer = _hdfext.array_int16_frompointer
array_int16_swigregister = _hdfext.array_int16_swigregister
array_int16_swigregister(array_int16)

def array_int16_frompointer(t):
    return _hdfext.array_int16_frompointer(t)
array_int16_frompointer = _hdfext.array_int16_frompointer

class array_uint16(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_uint16, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_uint16, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_uint16(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_uint16
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_uint16___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_uint16___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_uint16_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_uint16_frompointer)
    else:
        frompointer = _hdfext.array_uint16_frompointer
array_uint16_swigregister = _hdfext.array_uint16_swigregister
array_uint16_swigregister(array_uint16)

def array_uint16_frompointer(t):
    return _hdfext.array_uint16_frompointer(t)
array_uint16_frompointer = _hdfext.array_uint16_frompointer

class array_int32(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_int32, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_int32, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_int32(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_int32
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_int32___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_int32___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_int32_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_int32_frompointer)
    else:
        frompointer = _hdfext.array_int32_frompointer
array_int32_swigregister = _hdfext.array_int32_swigregister
array_int32_swigregister(array_int32)

def array_int32_frompointer(t):
    return _hdfext.array_int32_frompointer(t)
array_int32_frompointer = _hdfext.array_int32_frompointer

class array_uint32(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_uint32, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_uint32, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_uint32(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_uint32
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_uint32___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_uint32___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_uint32_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_uint32_frompointer)
    else:
        frompointer = _hdfext.array_uint32_frompointer
array_uint32_swigregister = _hdfext.array_uint32_swigregister
array_uint32_swigregister(array_uint32)

def array_uint32_frompointer(t):
    return _hdfext.array_uint32_frompointer(t)
array_uint32_frompointer = _hdfext.array_uint32_frompointer

class array_float32(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_float32, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_float32, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_float32(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_float32
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_float32___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_float32___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_float32_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_float32_frompointer)
    else:
        frompointer = _hdfext.array_float32_frompointer
array_float32_swigregister = _hdfext.array_float32_swigregister
array_float32_swigregister(array_float32)

def array_float32_frompointer(t):
    return _hdfext.array_float32_frompointer(t)
array_float32_frompointer = _hdfext.array_float32_frompointer

class array_float64(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, array_float64, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, array_float64, name)
    __repr__ = _swig_repr

    def __init__(self, nelements):
        this = _hdfext.new_array_float64(nelements)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _hdfext.delete_array_float64
    __del__ = lambda self: None

    def __getitem__(self, index):
        return _hdfext.array_float64___getitem__(self, index)

    def __setitem__(self, index, value):
        return _hdfext.array_float64___setitem__(self, index, value)

    def cast(self):
        return _hdfext.array_float64_cast(self)
    if _newclass:
        frompointer = staticmethod(_hdfext.array_float64_frompointer)
    else:
        frompointer = _hdfext.array_float64_frompointer
array_float64_swigregister = _hdfext.array_float64_swigregister
array_float64_swigregister(array_float64)

def array_float64_frompointer(t):
    return _hdfext.array_float64_frompointer(t)
array_float64_frompointer = _hdfext.array_float64_frompointer


def new_array_voidp(nelements):
    return _hdfext.new_array_voidp(nelements)
new_array_voidp = _hdfext.new_array_voidp

def delete_array_voidp(ary):
    return _hdfext.delete_array_voidp(ary)
delete_array_voidp = _hdfext.delete_array_voidp

def array_voidp_getitem(ary, index):
    return _hdfext.array_voidp_getitem(ary, index)
array_voidp_getitem = _hdfext.array_voidp_getitem

def array_voidp_setitem(ary, index, value):
    return _hdfext.array_voidp_setitem(ary, index, value)
array_voidp_setitem = _hdfext.array_voidp_setitem

def Hopen(filename, access_mode, num_dds_blocks):
    return _hdfext.Hopen(filename, access_mode, num_dds_blocks)
Hopen = _hdfext.Hopen

def Hclose(file_id):
    return _hdfext.Hclose(file_id)
Hclose = _hdfext.Hclose

def Hgetlibversion():
    return _hdfext.Hgetlibversion()
Hgetlibversion = _hdfext.Hgetlibversion

def Hgetfileversion(file_id):
    return _hdfext.Hgetfileversion(file_id)
Hgetfileversion = _hdfext.Hgetfileversion

def Hishdf(filename):
    return _hdfext.Hishdf(filename)
Hishdf = _hdfext.Hishdf

def HEvalue(error_stack_offset):
    return _hdfext.HEvalue(error_stack_offset)
HEvalue = _hdfext.HEvalue

def HEstring(error_code):
    return _hdfext.HEstring(error_code)
HEstring = _hdfext.HEstring

def _HEprint():
    return _hdfext._HEprint()
_HEprint = _hdfext._HEprint

def _SDreaddata_0(sds_id, data_type, start, edges, stride):
    return _hdfext._SDreaddata_0(sds_id, data_type, start, edges, stride)
_SDreaddata_0 = _hdfext._SDreaddata_0

def _SDwritedata_0(sds_id, data_type, start, edges, data, stride):
    return _hdfext._SDwritedata_0(sds_id, data_type, start, edges, data, stride)
_SDwritedata_0 = _hdfext._SDwritedata_0

def SDstart(filename, access_mode):
    return _hdfext.SDstart(filename, access_mode)
SDstart = _hdfext.SDstart

def SDcreate(sd_id, sds_name, data_type, rank, dim_sizes):
    return _hdfext.SDcreate(sd_id, sds_name, data_type, rank, dim_sizes)
SDcreate = _hdfext.SDcreate

def SDselect(sd_id, sds_index):
    return _hdfext.SDselect(sd_id, sds_index)
SDselect = _hdfext.SDselect

def SDendaccess(sds_id):
    return _hdfext.SDendaccess(sds_id)
SDendaccess = _hdfext.SDendaccess

def SDend(sd_id):
    return _hdfext.SDend(sd_id)
SDend = _hdfext.SDend

def SDfileinfo(sd_id):
    return _hdfext.SDfileinfo(sd_id)
SDfileinfo = _hdfext.SDfileinfo

def SDgetinfo(sds_id, buf):
    return _hdfext.SDgetinfo(sds_id, buf)
SDgetinfo = _hdfext.SDgetinfo

def SDcheckempty(sds_id):
    return _hdfext.SDcheckempty(sds_id)
SDcheckempty = _hdfext.SDcheckempty

def SDidtoref(sds_id):
    return _hdfext.SDidtoref(sds_id)
SDidtoref = _hdfext.SDidtoref

def SDiscoordvar(sds_id):
    return _hdfext.SDiscoordvar(sds_id)
SDiscoordvar = _hdfext.SDiscoordvar

def SDisrecord(sds_id):
    return _hdfext.SDisrecord(sds_id)
SDisrecord = _hdfext.SDisrecord

def SDnametoindex(sd_id, sds_name):
    return _hdfext.SDnametoindex(sd_id, sds_name)
SDnametoindex = _hdfext.SDnametoindex

def SDreftoindex(sd_id, sds_ref):
    return _hdfext.SDreftoindex(sd_id, sds_ref)
SDreftoindex = _hdfext.SDreftoindex

def SDdiminfo(dim_id):
    return _hdfext.SDdiminfo(dim_id)
SDdiminfo = _hdfext.SDdiminfo

def SDgetdimid(sds_id, dim_index):
    return _hdfext.SDgetdimid(sds_id, dim_index)
SDgetdimid = _hdfext.SDgetdimid

def SDsetdimname(dim_id, dim_name):
    return _hdfext.SDsetdimname(dim_id, dim_name)
SDsetdimname = _hdfext.SDsetdimname

def SDgetdimscale(dim_id, buf):
    return _hdfext.SDgetdimscale(dim_id, buf)
SDgetdimscale = _hdfext.SDgetdimscale

def SDsetdimscale(dim_id, n_values, data_type, buf):
    return _hdfext.SDsetdimscale(dim_id, n_values, data_type, buf)
SDsetdimscale = _hdfext.SDsetdimscale

def SDattrinfo(obj_id, attr_index):
    return _hdfext.SDattrinfo(obj_id, attr_index)
SDattrinfo = _hdfext.SDattrinfo

def SDfindattr(obj_id, attr_name):
    return _hdfext.SDfindattr(obj_id, attr_name)
SDfindattr = _hdfext.SDfindattr

def SDreadattr(obj_id, attr_index, buf):
    return _hdfext.SDreadattr(obj_id, attr_index, buf)
SDreadattr = _hdfext.SDreadattr

def SDsetattr(obj_id, attr_name, data_type, n_values, values):
    return _hdfext.SDsetattr(obj_id, attr_name, data_type, n_values, values)
SDsetattr = _hdfext.SDsetattr

def SDgetcal(sds_id):
    return _hdfext.SDgetcal(sds_id)
SDgetcal = _hdfext.SDgetcal

def SDgetdatastrs(sds_id, len):
    return _hdfext.SDgetdatastrs(sds_id, len)
SDgetdatastrs = _hdfext.SDgetdatastrs

def SDgetdimstrs(sds_id, len):
    return _hdfext.SDgetdimstrs(sds_id, len)
SDgetdimstrs = _hdfext.SDgetdimstrs

def SDgetfillvalue(sds_id, buf):
    return _hdfext.SDgetfillvalue(sds_id, buf)
SDgetfillvalue = _hdfext.SDgetfillvalue

def SDgetrange(sds_id, buf1, buf2):
    return _hdfext.SDgetrange(sds_id, buf1, buf2)
SDgetrange = _hdfext.SDgetrange

def SDsetcal(sds_id, cal, cal_error, offset, offset_err, data_type):
    return _hdfext.SDsetcal(sds_id, cal, cal_error, offset, offset_err, data_type)
SDsetcal = _hdfext.SDsetcal

def SDsetdatastrs(sds_id, label, unit, format, coord_system):
    return _hdfext.SDsetdatastrs(sds_id, label, unit, format, coord_system)
SDsetdatastrs = _hdfext.SDsetdatastrs

def SDsetdimstrs(sds_id, label, unit, format):
    return _hdfext.SDsetdimstrs(sds_id, label, unit, format)
SDsetdimstrs = _hdfext.SDsetdimstrs

def SDsetfillmode(sd_id, fill_mode):
    return _hdfext.SDsetfillmode(sd_id, fill_mode)
SDsetfillmode = _hdfext.SDsetfillmode

def SDsetfillvalue(sds_id, fill_val):
    return _hdfext.SDsetfillvalue(sds_id, fill_val)
SDsetfillvalue = _hdfext.SDsetfillvalue

def SDsetrange(sds_id, max, min):
    return _hdfext.SDsetrange(sds_id, max, min)
SDsetrange = _hdfext.SDsetrange

def _SDgetcompress(sds_id):
    return _hdfext._SDgetcompress(sds_id)
_SDgetcompress = _hdfext._SDgetcompress

def _SDsetcompress(sds_id, comp_type, value, v2):
    return _hdfext._SDsetcompress(sds_id, comp_type, value, v2)
_SDsetcompress = _hdfext._SDsetcompress

def SDsetexternalfile(sds_id, filename, offset):
    return _hdfext.SDsetexternalfile(sds_id, filename, offset)
SDsetexternalfile = _hdfext.SDsetexternalfile

def Vinitialize(file_id):
    return _hdfext.Vinitialize(file_id)
Vinitialize = _hdfext.Vinitialize

def VSattach(file_id, vdata_ref, vdata_access_mode):
    return _hdfext.VSattach(file_id, vdata_ref, vdata_access_mode)
VSattach = _hdfext.VSattach

def VSdetach(vdata_id):
    return _hdfext.VSdetach(vdata_id)
VSdetach = _hdfext.VSdetach

def Vfinish(file_id):
    return _hdfext.Vfinish(file_id)
Vfinish = _hdfext.Vfinish

def VHstoredata(file_id, fieldname, buf, n_records, data_type, vdata_name, vdata_class):
    return _hdfext.VHstoredata(file_id, fieldname, buf, n_records, data_type, vdata_name, vdata_class)
VHstoredata = _hdfext.VHstoredata

def VHstoredatam(file_id, fieldname, buf, n_records, data_type, vdata_name, vdata_class, order):
    return _hdfext.VHstoredatam(file_id, fieldname, buf, n_records, data_type, vdata_name, vdata_class, order)
VHstoredatam = _hdfext.VHstoredatam

def VSfdefine(vdata_id, fieldname, data_type, order):
    return _hdfext.VSfdefine(vdata_id, fieldname, data_type, order)
VSfdefine = _hdfext.VSfdefine

def VSsetfields(vdata_id, fieldname_list):
    return _hdfext.VSsetfields(vdata_id, fieldname_list)
VSsetfields = _hdfext.VSsetfields

def VSseek(vdata_id, record_index):
    return _hdfext.VSseek(vdata_id, record_index)
VSseek = _hdfext.VSseek

def VSread(vdata_id, databuf, n_records, interlace_mode):
    return _hdfext.VSread(vdata_id, databuf, n_records, interlace_mode)
VSread = _hdfext.VSread

def VSwrite(vdata_id, databuf, n_records, interlace_mode):
    return _hdfext.VSwrite(vdata_id, databuf, n_records, interlace_mode)
VSwrite = _hdfext.VSwrite

def VSfpack(vdata_id, action, fields_in_buf, buf, buf_size, n_records, fieldname_list, bufptrs):
    return _hdfext.VSfpack(vdata_id, action, fields_in_buf, buf, buf_size, n_records, fieldname_list, bufptrs)
VSfpack = _hdfext.VSfpack

def VSelts(vdata_id):
    return _hdfext.VSelts(vdata_id)
VSelts = _hdfext.VSelts

def VSgetclass(vdata_id):
    return _hdfext.VSgetclass(vdata_id)
VSgetclass = _hdfext.VSgetclass

def VSgetfields(vdata_id):
    return _hdfext.VSgetfields(vdata_id)
VSgetfields = _hdfext.VSgetfields

def VSgetinterlace(vdata_id):
    return _hdfext.VSgetinterlace(vdata_id)
VSgetinterlace = _hdfext.VSgetinterlace

def VSgetname(vdata_id):
    return _hdfext.VSgetname(vdata_id)
VSgetname = _hdfext.VSgetname

def VSsizeof(vdata_id, fieldname_list):
    return _hdfext.VSsizeof(vdata_id, fieldname_list)
VSsizeof = _hdfext.VSsizeof

def VSinquire(vdata_id):
    return _hdfext.VSinquire(vdata_id)
VSinquire = _hdfext.VSinquire

def VSQuerytag(vdata_id):
    return _hdfext.VSQuerytag(vdata_id)
VSQuerytag = _hdfext.VSQuerytag

def VSQueryref(vdata_id):
    return _hdfext.VSQueryref(vdata_id)
VSQueryref = _hdfext.VSQueryref

def VSfindex(vdata_id, field_name):
    return _hdfext.VSfindex(vdata_id, field_name)
VSfindex = _hdfext.VSfindex

def VSisattr(vdta_id):
    return _hdfext.VSisattr(vdta_id)
VSisattr = _hdfext.VSisattr

def VFnfields(vdata_id):
    return _hdfext.VFnfields(vdata_id)
VFnfields = _hdfext.VFnfields

def VFfieldtype(vdata_id, field_index):
    return _hdfext.VFfieldtype(vdata_id, field_index)
VFfieldtype = _hdfext.VFfieldtype

def VFfieldname(vdata_id, field_index):
    return _hdfext.VFfieldname(vdata_id, field_index)
VFfieldname = _hdfext.VFfieldname

def VFfieldesize(vdata_id, field_index):
    return _hdfext.VFfieldesize(vdata_id, field_index)
VFfieldesize = _hdfext.VFfieldesize

def VFfieldisize(vdata_id, field_index):
    return _hdfext.VFfieldisize(vdata_id, field_index)
VFfieldisize = _hdfext.VFfieldisize

def VFfieldorder(vdata_id, field_index):
    return _hdfext.VFfieldorder(vdata_id, field_index)
VFfieldorder = _hdfext.VFfieldorder

def VSfind(file_id, vdata_name):
    return _hdfext.VSfind(file_id, vdata_name)
VSfind = _hdfext.VSfind

def VSgetid(file_id, vdata_ref):
    return _hdfext.VSgetid(file_id, vdata_ref)
VSgetid = _hdfext.VSgetid

def VSfexist(vdata_id, fieldname_list):
    return _hdfext.VSfexist(vdata_id, fieldname_list)
VSfexist = _hdfext.VSfexist

def VSsetclass(vdata_id, vdata_class):
    return _hdfext.VSsetclass(vdata_id, vdata_class)
VSsetclass = _hdfext.VSsetclass

def VSsetname(vdata_id, vdata_name):
    return _hdfext.VSsetname(vdata_id, vdata_name)
VSsetname = _hdfext.VSsetname

def VSsetinterlace(vdata_id, interlace_mode):
    return _hdfext.VSsetinterlace(vdata_id, interlace_mode)
VSsetinterlace = _hdfext.VSsetinterlace

def VSsetattr(vdata_id, field_index, attr_name, data_type, n_values, values):
    return _hdfext.VSsetattr(vdata_id, field_index, attr_name, data_type, n_values, values)
VSsetattr = _hdfext.VSsetattr

def VSgetattr(vdata_id, field_index, attr_index, buf):
    return _hdfext.VSgetattr(vdata_id, field_index, attr_index, buf)
VSgetattr = _hdfext.VSgetattr

def VSfnattrs(vdata_id, field_index):
    return _hdfext.VSfnattrs(vdata_id, field_index)
VSfnattrs = _hdfext.VSfnattrs

def VSnattrs(vdata_id):
    return _hdfext.VSnattrs(vdata_id)
VSnattrs = _hdfext.VSnattrs

def VSattrinfo(vdata_id, field_index, attr_index):
    return _hdfext.VSattrinfo(vdata_id, field_index, attr_index)
VSattrinfo = _hdfext.VSattrinfo

def VSfindattr(vdata_id, field_index, attr_name):
    return _hdfext.VSfindattr(vdata_id, field_index, attr_name)
VSfindattr = _hdfext.VSfindattr

def Vattach(file_id, vgroup_ref, vg_access_mode):
    return _hdfext.Vattach(file_id, vgroup_ref, vg_access_mode)
Vattach = _hdfext.Vattach

def Vdetach(vgroup_id):
    return _hdfext.Vdetach(vgroup_id)
Vdetach = _hdfext.Vdetach

def Vgetname(vgroup_id):
    return _hdfext.Vgetname(vgroup_id)
Vgetname = _hdfext.Vgetname

def Vsetname(vgroup_id, vgroup_name):
    return _hdfext.Vsetname(vgroup_id, vgroup_name)
Vsetname = _hdfext.Vsetname

def Vgetclass(vgroup_id):
    return _hdfext.Vgetclass(vgroup_id)
Vgetclass = _hdfext.Vgetclass

def Vsetclass(vgroup_id, vgroup_class):
    return _hdfext.Vsetclass(vgroup_id, vgroup_class)
Vsetclass = _hdfext.Vsetclass

def Vfind(file_id, vgroup_name):
    return _hdfext.Vfind(file_id, vgroup_name)
Vfind = _hdfext.Vfind

def Vfindclass(file_id, vgroup_class):
    return _hdfext.Vfindclass(file_id, vgroup_class)
Vfindclass = _hdfext.Vfindclass

def Vinsert(vgroup_id, v_id):
    return _hdfext.Vinsert(vgroup_id, v_id)
Vinsert = _hdfext.Vinsert

def Vaddtagref(vgroup_id, obj_tag, obj_ref):
    return _hdfext.Vaddtagref(vgroup_id, obj_tag, obj_ref)
Vaddtagref = _hdfext.Vaddtagref

def Vdeletetagref(vgroup_id, obj_tag, obj_ref):
    return _hdfext.Vdeletetagref(vgroup_id, obj_tag, obj_ref)
Vdeletetagref = _hdfext.Vdeletetagref

def Vdelete(file_id, vgroup_id):
    return _hdfext.Vdelete(file_id, vgroup_id)
Vdelete = _hdfext.Vdelete

def VQueryref(vgroup_id):
    return _hdfext.VQueryref(vgroup_id)
VQueryref = _hdfext.VQueryref

def VQuerytag(vgroup_id):
    return _hdfext.VQuerytag(vgroup_id)
VQuerytag = _hdfext.VQuerytag

def Vntagrefs(vgroup_id):
    return _hdfext.Vntagrefs(vgroup_id)
Vntagrefs = _hdfext.Vntagrefs

def Vgettagref(vgroup_id, index):
    return _hdfext.Vgettagref(vgroup_id, index)
Vgettagref = _hdfext.Vgettagref

def Vgetversion(vgroup_id):
    return _hdfext.Vgetversion(vgroup_id)
Vgetversion = _hdfext.Vgetversion

def Vgettagrefs(vgroup_id, tag_attay, ref_array, maxsize):
    return _hdfext.Vgettagrefs(vgroup_id, tag_attay, ref_array, maxsize)
Vgettagrefs = _hdfext.Vgettagrefs

def Vgetid(file_id, vgroup_ref):
    return _hdfext.Vgetid(file_id, vgroup_ref)
Vgetid = _hdfext.Vgetid

def Vinqtagref(vgroup_id, tag, ref):
    return _hdfext.Vinqtagref(vgroup_id, tag, ref)
Vinqtagref = _hdfext.Vinqtagref

def Visvg(vgroup_id, obj_ref):
    return _hdfext.Visvg(vgroup_id, obj_ref)
Visvg = _hdfext.Visvg

def Visvs(vgroup_id, obj_ref):
    return _hdfext.Visvs(vgroup_id, obj_ref)
Visvs = _hdfext.Visvs

def Vnrefs(vgroup_id, tag_type):
    return _hdfext.Vnrefs(vgroup_id, tag_type)
Vnrefs = _hdfext.Vnrefs

def Vfindattr(vgroup_id, attr_name):
    return _hdfext.Vfindattr(vgroup_id, attr_name)
Vfindattr = _hdfext.Vfindattr

def Vgetattr(vdata_id, attr_index, buf):
    return _hdfext.Vgetattr(vdata_id, attr_index, buf)
Vgetattr = _hdfext.Vgetattr

def Vsetattr(vgroup_id, attr_name, data_type, n_values, values):
    return _hdfext.Vsetattr(vgroup_id, attr_name, data_type, n_values, values)
Vsetattr = _hdfext.Vsetattr

def Vattrinfo(vgroup_id, attr_index):
    return _hdfext.Vattrinfo(vgroup_id, attr_index)
Vattrinfo = _hdfext.Vattrinfo

def Vnattrs(vgroup_id):
    return _hdfext.Vnattrs(vgroup_id)
Vnattrs = _hdfext.Vnattrs
# This file is compatible with both classic and new-style classes.





#!/usr/bin/env python

from __future__ import print_function

# Generate test HDF files using different compression configurations,
# and validate each resulting file to make sure its contents is OK.
# Adapted from example:
# "https://support.hdfgroup.org/doc_resource/SZIP/h4_examples/szip32.c".
# A bigger dataset is defined to better show the size reduction achieved
# by the SZIP compression.
# Note that when applied to HDF4 SDS, the word "pixels" as used inside
# the SZIP documentation should really be understood as a "data element",
# eg a cell value inside a multidimensional array.
#
# On our systems, the program produced the following file sizes :
#
# $ ls -l *.hdf
#    -rw-r--r-- 1 root root  53389 Jun 29 14:20 SDS.COMP_DEFLATE.1.hdf
#    -rw-r--r-- 1 root root  56524 Jun 29 14:24 SDS.COMP_DEFLATE.2.hdf
#    -rw-r--r-- 1 root root  60069 Jun 29 14:24 SDS.COMP_DEFLATE.3.hdf
#    -rw-r--r-- 1 root root  59725 Jun 29 14:24 SDS.COMP_DEFLATE.4.hdf
#    -rw-r--r-- 1 root root  59884 Jun 29 14:24 SDS.COMP_DEFLATE.5.hdf
#    -rw-r--r-- 1 root root  58596 Jun 29 14:24 SDS.COMP_DEFLATE.6.hdf
#    -rw-r--r-- 1 root root  58450 Jun 29 14:24 SDS.COMP_DEFLATE.7.hdf
#    -rw-r--r-- 1 root root  58437 Jun 29 14:24 SDS.COMP_DEFLATE.8.hdf
#    -rw-r--r-- 1 root root  58446 Jun 29 14:24 SDS.COMP_DEFLATE.9.hdf
#    -rw-r--r-- 1 root root 102920 Jun 29 14:20 SDS.COMP_NONE.hdf
#    -rw-r--r-- 1 root root 103162 Jun 29 14:20 SDS.COMP_RLE.hdf
#    -rw-r--r-- 1 root root  60277 Jun 29 14:20 SDS.COMP_SKPHUFF.2.hdf
#    -rw-r--r-- 1 root root  52085 Jun 29 14:20 SDS.COMP_SKPHUFF.4.hdf
#    -rw-r--r-- 1 root root  52085 Jun 29 14:20 SDS.COMP_SKPHUFF.8.hdf
#    -rw-r--r-- 1 root root  71039 Jun 29 14:20 SDS.COMP_SZIP.EC.16.hdf
#    -rw-r--r-- 1 root root  79053 Jun 29 14:20 SDS.COMP_SZIP.EC.32.hdf
#    -rw-r--r-- 1 root root  66636 Jun 29 14:20 SDS.COMP_SZIP.EC.4.hdf
#    -rw-r--r-- 1 root root  66984 Jun 29 14:20 SDS.COMP_SZIP.EC.8.hdf
#    -rw-r--r-- 1 root root  39835 Jun 29 14:20 SDS.COMP_SZIP.NN.16.hdf
#    -rw-r--r-- 1 root root  44554 Jun 29 14:20 SDS.COMP_SZIP.NN.32.hdf
#    -rw-r--r-- 1 root root  38371 Jun 29 14:20 SDS.COMP_SZIP.NN.4.hdf
#    -rw-r--r-- 1 root root  38092 Jun 29 14:20 SDS.COMP_SZIP.NN.8.hdf
#
# For the chosen data set, the best results were attained using
# SZIP compression with NN compression scheme and 8 pixels per block.
# Mileage will vary with the data set used.

import sys
import os.path

from pyhdf.SD import *
import numpy

# Array shape and data type.
LENGTH      = 250
WIDTH       = 100
NUMPY_DATATYPE = numpy.int32
HDF_DATATYPE   = SDC.INT32

def doCompress(compType, value=0, v2=0):
    """Create and validate an HDF file using a compression scheme
    specified by the parameters"""

    # Build a significant file name
    if compType == SDC.COMP_NONE:
        fileName = "SDS.COMP_NONE"
    elif compType == SDC.COMP_RLE:
        fileName = "SDS.COMP_RLE"
    elif compType == SDC.COMP_SKPHUFF:
        fileName = "SDS.COMP_SKPHUFF.%d" % value
    elif compType == SDC.COMP_DEFLATE:
        fileName = "SDS.COMP_DEFLATE.%d" % value
    elif compType == SDC.COMP_SZIP:
        fileName = "SDS.COMP_SZIP"
        if value == SDC.COMP_SZIP_NN:
            fileName += ".NN"
        elif value == SDC.COMP_SZIP_EC:
            fileName += ".EC"
        else:
            print("illegal value")
            sys.exit(1)
        fileName += ".%s" % v2
    else:
        print("illegal compType")
        sys.exit(1)
    fileName += ".hdf"

    SDS_NAME    = "Data"

    fill_value  = 0

    #LENGTH      = 9
    #WIDTH       = 6
    #
    #data = numpy.array(  ((100,100,200,200,300,400),
    #                      (100,100,200,200,300,400),
    #                      (100,100,200,200,300,400),
    #                      (300,300,  0,400,300,400),
    #                      (300,300,  0,400,300,400),
    #                      (300,300,  0,400,300,400),
    #                      (0,  0,600,600,300,400),
    #                      (500,500,600,600,300,400),
    #                      (0,  0,600,600,300,400)), NUMPY_DATATYPE)

    # The above dataset is used in the original NCSA example.
    # It is too small to show a significant size reduction after
    # compression. The following is used for a more realistic example.
    data = numpy.zeros((LENGTH, WIDTH), NUMPY_DATATYPE)
    for i in range(LENGTH):
        for j in range(WIDTH):
            data[i,j] = (i+j)*(i-j)

    # Create HDF file, wiping it out it it already exists.
    sd_id = SD(fileName, SDC.WRITE | SDC.CREATE | SDC.TRUNC)

    # Create dataset.
    sds_id = sd_id.create(SDS_NAME, HDF_DATATYPE, (LENGTH, WIDTH))

    # Fill dataset will fill value.
    sds_id.setfillvalue(0)

    # Apply compression.
    try:
        sds_id.setcompress(compType,        # compression type
                           value, v2)         # args depend on compression type
    except HDF4Error as msg:
        print(("Error compressing the dataset with params: "
              "(%d,%d,%d) : %s" % (compType, value, v2, msg)))
        sds_id.endaccess()
        sd_id.end()
        os.remove(fileName)
        return

    # Load data in the dataset.
    sds_id[:] = data

    # Close dataset.
    sds_id.endaccess()

    # Close hdf file to flush compressed data.
    sd_id.end()

    # Verify compressed data.
    # ######################

    # Reopen file and select first dataset.
    sd_id = SD(fileName, SDC.READ)
    sds_id = sd_id.select(0)

    # Obtain compression info.
    compInfo = sds_id.getcompress()
    compType = compInfo[0]
    print("file : %s" % fileName)
    print("  size = %d" % os.path.getsize(fileName))
    if compType == SDC.COMP_NONE:
        print("  compType =  COMP_NONE")
    elif compType == SDC.COMP_RLE:
        print("  compType =  COMP_RLE")
    elif compType == SDC.COMP_SKPHUFF:
        print("  compType = COMP_SKPHUFF")
        print("  dataSize = %d" % compInfo[1])
    elif compType == SDC.COMP_DEFLATE:
        print("  compType = COMP_DEFLATE (GZIP)")
        print("  level = %d" % compInfo[1])
    elif compType == SDC.COMP_SZIP:
        print("  compType = COMP_SZIP")
        optionMask  = compInfo[1]
        if optionMask & SDC.COMP_SZIP_NN:
            print("  encoding scheme = NN")
        elif optionMask & SDC.COMP_SZIP_EC:
            print("  encoding scheme = EC")
        else:
            print("  unknown encoding scheme")
            sys.exit(1)
        pixelsPerBlock, pixelsPerScanline, bitsPerPixel, pixels  = compInfo[2:]
        print("  pixelsPerBlock = %d" % pixelsPerBlock)
        print("  pixelsPerScanline = %d" % pixelsPerScanline)
        print("  bitsPerPixel = %d" % bitsPerPixel)
        print("  pixels = %d" % pixels)
    else:
        print("  unknown compression type")
        sys.exit(1)

    # Read dataset contents.
    out_data = sds_id[:]

    # Compare with original data.
    num_errs = 0
    for i in range(LENGTH):
        for j in range(WIDTH):
            if data[i,j] != out_data[i,j]:
                print("bad value at %d,%d expected: %d got: %d" \
                      % (i,j,data[i,j],out_data[i,j]))
                num_errs += 1

    # Close dataset and hdf file.
    sds_id.endaccess()
    sd_id.end()

    if num_errs == 0:
        print("  file validated")
    else:
        print("  file invalid : %d errors" % num_errs)
    print("")

# Try different compression configurations in turn.

# All the following calls will fail with a "Cannot execute" exception if pyhdf
# was installed with the NOCOMPRESS macro set.

# No compression
print("no compression")
doCompress(SDC.COMP_NONE)

# RLE compression
print("run-length encoding")
doCompress(SDC.COMP_RLE)

# Skipping-Huffman compression.
print("Skipping-Huffman encoding")
for size in 2,4,8:
    doCompress(SDC.COMP_SKPHUFF, size)   # size in bytes of the data elements

# Gzip compression
print("GZIP compression")
for level in 1,2,3,4,5,6,7,8,9:
    doCompress(SDC.COMP_DEFLATE, level)   # compression level, from 1 to 9

# SZIP compression
# Those calls will fail with an "Encoder not available" exception if
# pyhdf was installed with the NOSZIP macro set.
print("SZIP compression")
for scheme in SDC.COMP_SZIP_NN, SDC.COMP_SZIP_EC:
    for ppb in 4,8,16,32:
        doCompress(SDC.COMP_SZIP, scheme, ppb)  # scheme, pixels per block
